"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var baseplatform_1 = require("./baseplatform");
var emu_1 = require("./emu");
var StateRecorderImpl = /** @class */ (function () {
    function StateRecorderImpl(platform) {
        this.checkpointInterval = 60;
        this.maxCheckpoints = 120;
        this.reset();
        this.platform = platform;
    }
    StateRecorderImpl.prototype.reset = function () {
        this.checkpoints = [];
        this.framerecs = [];
        this.frameCount = 0;
        this.lastSeekFrame = 0;
        if (this.callbackStateChanged)
            this.callbackStateChanged();
    };
    StateRecorderImpl.prototype.frameRequested = function () {
        var controls = {
            controls: this.platform.saveControlsState(),
            seed: emu_1.getNoiseSeed()
        };
        var requested = false;
        // are we replaying? then we don't need to save a frame, just replace controls
        if (this.lastSeekFrame < this.frameCount) {
            this.loadControls(this.lastSeekFrame);
        }
        else {
            // record the control state, if available
            if (this.platform.saveControlsState) {
                this.framerecs.push(controls);
            }
            // time to save next frame?
            requested = (this.frameCount++ % this.checkpointInterval) == 0;
        }
        this.lastSeekFrame++;
        if (this.callbackStateChanged)
            this.callbackStateChanged();
        return requested;
    };
    StateRecorderImpl.prototype.numFrames = function () {
        return this.frameCount;
    };
    StateRecorderImpl.prototype.currentFrame = function () {
        return this.lastSeekFrame;
    };
    StateRecorderImpl.prototype.recordFrame = function (state) {
        this.checkpoints.push(state);
        if (this.callbackNewCheckpoint)
            this.callbackNewCheckpoint(state);
        // checkpoints full?
        if (this.checkpoints.length > this.maxCheckpoints) {
            this.checkpoints.shift(); // remove 1st checkpoint
            this.framerecs = this.framerecs.slice(this.checkpointInterval);
            this.lastSeekFrame -= this.checkpointInterval;
            this.frameCount -= this.checkpointInterval;
            if (this.callbackStateChanged)
                this.callbackStateChanged();
        }
    };
    StateRecorderImpl.prototype.getStateAtOrBefore = function (frame) {
        var bufidx = Math.floor(frame / this.checkpointInterval);
        var foundidx = bufidx < this.checkpoints.length ? bufidx : this.checkpoints.length - 1;
        var foundframe = foundidx * this.checkpointInterval;
        return { frame: foundframe, state: this.checkpoints[foundidx] };
    };
    StateRecorderImpl.prototype.loadFrame = function (seekframe) {
        if (seekframe == this.lastSeekFrame)
            return seekframe; // already set to this frame
        // TODO: what if < 1?
        var _a = this.getStateAtOrBefore(seekframe - 1), frame = _a.frame, state = _a.state;
        if (state) {
            this.platform.pause();
            this.platform.loadState(state);
            while (frame < seekframe) {
                if (frame < this.framerecs.length) {
                    this.loadControls(frame);
                }
                frame++;
                this.platform.advance(frame < seekframe); // TODO: infinite loop?
            }
            this.lastSeekFrame = seekframe;
            return seekframe;
        }
        else {
            return 0;
        }
    };
    StateRecorderImpl.prototype.loadControls = function (frame) {
        if (this.platform.loadControlsState)
            this.platform.loadControlsState(this.framerecs[frame].controls);
        emu_1.setNoiseSeed(this.framerecs[frame].seed);
    };
    StateRecorderImpl.prototype.getLastCheckpoint = function () {
        return this.checkpoints.length && this.checkpoints[this.checkpoints.length - 1];
    };
    return StateRecorderImpl;
}());
exports.StateRecorderImpl = StateRecorderImpl;
// PROFILER
var PROFOP_READ = 0x100000;
var PROFOP_WRITE = 0x200000;
var PROFOP_INTERRUPT = 0x400000;
var EmuProfilerImpl = /** @class */ (function () {
    function EmuProfilerImpl(platform) {
        this.frame = null;
        this.output = { frame: null };
        this.i = 0;
        this.lastsl = 9999;
        this.starti = 0;
        this.platform = platform;
    }
    EmuProfilerImpl.prototype.start = function () {
        var _this = this;
        if (this.platform instanceof baseplatform_1.BasePlatform)
            this.platform.profiler = this;
        this.platform.setBreakpoint('profile', function () {
            var c = _this.platform.getCPUState();
            _this.log(c.EPC || c.PC);
            return false; // profile forever
        });
        this.output = { frame: null };
        return this.output;
    };
    EmuProfilerImpl.prototype.log = function (op) {
        var sl = this.platform.getRasterScanline();
        if (sl != this.lastsl) {
            if (this.frame) {
                this.frame.lines.push({ start: this.starti, end: this.i - 1 });
            }
            if (sl < this.lastsl) {
                this.output.frame = this.frame;
                this.frame = { iptab: new Uint32Array(0x8000), lines: [] }; // TODO: const
                this.i = 0;
            }
            this.starti = this.i;
            this.lastsl = sl;
        }
        this.frame.iptab[this.i++] = op;
    };
    EmuProfilerImpl.prototype.stop = function () {
        this.platform.clearBreakpoint('profile');
        if (this.platform instanceof baseplatform_1.BasePlatform)
            this.platform.profiler = null;
    };
    // TODO?
    EmuProfilerImpl.prototype.logRead = function (a) {
        this.log(a | PROFOP_READ);
    };
    EmuProfilerImpl.prototype.logWrite = function (a) {
        this.log(a | PROFOP_WRITE);
    };
    EmuProfilerImpl.prototype.logInterrupt = function (a) {
        this.log(a | PROFOP_INTERRUPT);
    };
    return EmuProfilerImpl;
}());
exports.EmuProfilerImpl = EmuProfilerImpl;
var ProbeFlags;
(function (ProbeFlags) {
    ProbeFlags[ProbeFlags["CLOCKS"] = 0] = "CLOCKS";
    ProbeFlags[ProbeFlags["EXECUTE"] = 16777216] = "EXECUTE";
    ProbeFlags[ProbeFlags["MEM_READ"] = 33554432] = "MEM_READ";
    ProbeFlags[ProbeFlags["MEM_WRITE"] = 50331648] = "MEM_WRITE";
    ProbeFlags[ProbeFlags["IO_READ"] = 67108864] = "IO_READ";
    ProbeFlags[ProbeFlags["IO_WRITE"] = 83886080] = "IO_WRITE";
    ProbeFlags[ProbeFlags["VRAM_READ"] = 100663296] = "VRAM_READ";
    ProbeFlags[ProbeFlags["VRAM_WRITE"] = 117440512] = "VRAM_WRITE";
    ProbeFlags[ProbeFlags["INTERRUPT"] = 134217728] = "INTERRUPT";
    ProbeFlags[ProbeFlags["SCANLINE"] = 2113929216] = "SCANLINE";
    ProbeFlags[ProbeFlags["FRAME"] = 2130706432] = "FRAME";
})(ProbeFlags = exports.ProbeFlags || (exports.ProbeFlags = {}));
var ProbeFrame = /** @class */ (function () {
    function ProbeFrame() {
    }
    return ProbeFrame;
}());
var ProbeRecorder = /** @class */ (function () {
    function ProbeRecorder(m) {
        this.buf = new Uint32Array(0x100000);
        this.idx = 0;
        this.fclk = 0;
        this.sl = 0;
        this.singleFrame = true;
        this.m = m;
    }
    ProbeRecorder.prototype.start = function () {
        this.m.connectProbe(this);
    };
    ProbeRecorder.prototype.stop = function () {
        this.m.connectProbe(null);
    };
    ProbeRecorder.prototype.reset = function () {
        this.idx = 0;
    };
    ProbeRecorder.prototype.log = function (a) {
        // TODO: coalesce READ and EXECUTE
        if (this.idx >= this.buf.length)
            return;
        this.buf[this.idx++] = a;
    };
    ProbeRecorder.prototype.relog = function (a) {
        this.buf[this.idx - 1] = a;
    };
    ProbeRecorder.prototype.lastOp = function () {
        if (this.idx > 0)
            return this.buf[this.idx - 1] & 0xff000000;
        else
            return -1;
    };
    ProbeRecorder.prototype.lastAddr = function () {
        if (this.idx > 0)
            return this.buf[this.idx - 1] & 0xffffff;
        else
            return -1;
    };
    ProbeRecorder.prototype.logClocks = function (clocks) {
        if (clocks > 0) {
            this.fclk += clocks;
            if (this.lastOp() == ProbeFlags.CLOCKS)
                this.relog((this.lastAddr() + clocks) | ProbeFlags.CLOCKS); // coalesce clocks
            else
                this.log(clocks | ProbeFlags.CLOCKS);
        }
    };
    ProbeRecorder.prototype.logNewScanline = function () {
        this.log(ProbeFlags.SCANLINE);
        this.sl++;
    };
    ProbeRecorder.prototype.logNewFrame = function () {
        this.log(ProbeFlags.FRAME);
        this.sl = 0;
        if (this.singleFrame)
            this.reset();
    };
    ProbeRecorder.prototype.logExecute = function (address) {
        this.log(address | ProbeFlags.EXECUTE);
    };
    ProbeRecorder.prototype.logInterrupt = function (type) {
        this.log(type | ProbeFlags.INTERRUPT);
    };
    ProbeRecorder.prototype.logRead = function (address, value) {
        this.log(address | ProbeFlags.MEM_READ);
    };
    ProbeRecorder.prototype.logWrite = function (address, value) {
        this.log(address | ProbeFlags.MEM_WRITE);
    };
    ProbeRecorder.prototype.logIORead = function (address, value) {
        this.log(address | ProbeFlags.IO_READ);
    };
    ProbeRecorder.prototype.logIOWrite = function (address, value) {
        this.log(address | ProbeFlags.IO_WRITE);
    };
    ProbeRecorder.prototype.logVRAMRead = function (address, value) {
        this.log(address | ProbeFlags.VRAM_READ);
    };
    ProbeRecorder.prototype.logVRAMWrite = function (address, value) {
        this.log(address | ProbeFlags.VRAM_WRITE);
    };
    return ProbeRecorder;
}());
exports.ProbeRecorder = ProbeRecorder;
// TODO: handle runToVsync() without erasing entire frame
//# sourceMappingURL=recorder.js.map